unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    MasterPanel: TPanel;
    ScanButton: TButton;
    ErrLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure ScanButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses
  AIOUSB, TintProgresses;

var
  ChannelPanel: array[0..127] of TPanel;
  ChannelLabel: array[0..127] of TLabel;
  ChannelProgress: array[0..127] of TTintProgress;
  DI: LongWord;

procedure TForm1.FormCreate(Sender: TObject);
var
  I: Integer;
  Err, L, PID: LongWord;
  V: Double;
  Config: array of Byte;
  RangeCode: Byte;

begin
  ScanButton.Align := alLeft;
  for I := 0 to 127 do begin
    ChannelPanel[I] := TPanel.Create(Self);
    with ChannelPanel[I] do begin
      BevelOuter := bvNone;
      Top := MasterPanel.Height + 16 * I;
      Height := 16;
      Align := alTop;
      Parent := Self;
    end;
    ChannelLabel[I] := TLabel.Create(Self);
    with ChannelLabel[I] do begin
      AutoSize := False;
      Width := 80;
      Align := alLeft;
      Parent := ChannelPanel[I];
    end;
    ChannelProgress[I] := TTintProgress.Create(Self);
    with ChannelProgress[I] do begin
      Align := alClient;
      Parent := ChannelPanel[I];
    end;
  end;

  DI := ResolveDeviceIndex(diOnly);
  if DI = diNone then begin
    ErrLabel.Caption := 'Error: Failed to resolve diOnly.';
    Exit;
  end;

  Err := QueryDeviceInfo(DI, @PID, nil, nil, nil, nil);
  if Err <> ERROR_SUCCESS then begin
    ErrLabel.Caption := 'Error: QueryDeviceInfo returned error ' + IntToStr(Err) + '.';
    Exit;
  end;

  case PID of
    $8059..$805D, $8159..$815D: ;
    else begin
      ErrLabel.Caption := 'Error: Device found(PID ' + IntToHex(PID, 4) + ') is not a USB-AIxx-128x.';
      Exit;
    end;
  end;

  ADC_SetCal(DI, ':AUTO:');

  L := 21;
  SetLength(Config, L);
  for I := 0 to High(Config) do Config[I] := 0;

  Config[0] := $04;
  for I := 1 to 15 do Config[I] := $01;
  Config[$13] := 7; //Oversample +7. Max oversample for BG timer acq is +30.
  //Config bytes 11, 12, and 14 hex are set by the init.
  ADC_SetConfig(DI, @Config[0], L);

  Err := ADC_InitFastITScanV(DI);
  if Err <> ERROR_SUCCESS then begin
    ErrLabel.Caption := 'Error: ADC_InitFastITScanV returned error ' + IntToStr(Err) + '.';
    Exit;
  end;

  for I := 0 to 127 do begin
    RangeCode := Config[I div 8];
    V := 1;
    if (RangeCode and 2) = 0 then V := V * 2;
    if (RangeCode and 4) = 0 then V := V * 5;
    if (RangeCode and 1) <> 0 then
      ChannelProgress[I].Min := Round(-1000 * V)
    else
      ChannelProgress[I].Min := 0
    ;
    ChannelProgress[I].Max := Round(+1000 * V);
    ChannelProgress[I].Position := 0;
  end;

  ErrLabel.Caption := 'OK: Found PID ' + IntToHex(PID, 4) + ' at index ' + IntToStr(DI) + '.';
end;

procedure TForm1.ScanButtonClick(Sender: TObject);
var
  I: Integer;
  Err: LongWord;
  ADBuf: array of Double;
  V: Double;

begin
  SetLength(ADBuf, 128);
  for I := 0 to High(ADBuf) do ADBuf[I] := 42;

  Err := ADC_GetFastITScanV(DI, @ADBuf[0]);
  if Err <> ERROR_SUCCESS then begin
    ErrLabel.Caption := 'Error: ADC_InitFastITScanV returned error ' + IntToStr(Err) + '.';
    Exit;
  end;

  for I := 0 to 127 do begin
    V := ADBuf[I];
    if V = 42 then begin
      ChannelLabel[I].Caption := 'Ch ' + IntToStr(I) + ': ???';
      ChannelProgress[I].Position := 0;
    end
    else begin
      ChannelLabel[I].Caption := 'Ch ' + IntToStr(I) + ': ' + Format('%g', [ Round(V * 1000)/1000 ]);
      ChannelProgress[I].Position := Round(1000 * V);
    end;
  end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var
  Err: LongWord;

begin
  //Optional, puts things back the way they were before ADC_InitFastITScanV.
  Err := ADC_ResetFastITScanV(DI);
end;

end.
