unit TintProgresses;

interface

uses
  Windows, SysUtils, Classes, Controls, Graphics;

type
  TOrientation = (toUp, toDown, toLeft, toRight);

  TTintProgress = class(TGraphicControl)
  protected
    FMin: Integer;
    FMax: Integer;
    FPosition: Integer;
    FOrientation: TOrientation;
    FBorderWidth: Integer;
    FTint: array[0..4] of TColor;
    FSimpleTint: Boolean;
    procedure SetMin(Value: Integer);
    procedure SetMax(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure SetOrientation(Value: TOrientation);
    procedure SetBorderWidth(Value: Integer);
    procedure SetTint(Value: TColor);
    function GetTints(Index: Integer): TColor;
    procedure SetTints(Index: Integer; Value: TColor);
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Min: Integer read FMin write SetMin;
    property Max: Integer read FMax write SetMax;
    property Orientation: TOrientation read FOrientation write SetOrientation default toRight;
    property Position: Integer read FPosition write SetPosition default 0;
    property BorderWidth: Integer read FBorderWidth write SetBorderWidth default 1;
    property Tint: TColor read FTint[2] write SetTint default $FF0000;
  public
    property Tints[Index: Integer]: TColor read GetTints write SetTints;
  published
    property Align;
    property Anchors;
    property Color;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Hint;
    property Constraints;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;





implementation

uses
  Math;

type
  TRGBFColor = packed record
  case Boolean of
    False: (R, G, B, F: Byte);
    True: (Color: TColor);
  end;

function Fade(A, B: TColor; F: Single): TColor;
var
  AB: TRGBFColor absolute A;
  BB: TRGBFColor absolute B;
  RB: TRGBFColor absolute Result;
  nF: Single;

begin
  nF := 1 - F;
  RB.R := Round(AB.R * nF + BB.R * F);
  RB.G := Round(AB.G * nF + BB.G * F);
  RB.B := Round(AB.B * nF + BB.B * F);
  RB.F := 0;
end;

{ TTintProgress }

constructor TTintProgress.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 150;
  Height := GetSystemMetrics(SM_CYVSCROLL);
  FMin := 0;
  FMax := 100;
  FOrientation := toRight;
  FBorderWidth := 1;
  Tint := $FF0000;
end;

function TTintProgress.GetTints(Index: Integer): TColor;
begin
  if (Index < 0) or (Index > 4) then
    raise ERangeError.Create('Index ' + IntToStr(Index) + ' is invalid.')
  ;
  Result := FTint[Index];
end;

procedure TTintProgress.Paint;
var
  R: TRect;

  procedure AddEdge;
  begin
    Dec(R.Right);
    Dec(R.Bottom);
    Canvas.Pen.Color := clBtnShadow;
    Canvas.MoveTo(R.Left    , R.Top    );
    Canvas.LineTo(R.Right   , R.Top    );
    Canvas.MoveTo(R.Left    , R.Top + 1);
    Canvas.LineTo(R.Left    , R.Bottom );

    Canvas.Pen.Color := clBtnHighlight;
    Canvas.MoveTo(R.Right    , R.Top + 1);
    Canvas.LineTo(R.Right    , R.Bottom );
    Canvas.MoveTo(R.Left  + 1, R.Bottom );
    Canvas.LineTo(R.Right + 1, R.Bottom );
    Canvas.Pixels[R.Left     , R.Bottom ] := clBtnFace;
    Canvas.Pixels[R.Right    , R.Top    ] := clBtnFace;
    Inc(R.Left);
    Inc(R.Top);
  end;

var
  I, D, xD, yD: Integer;
  F: Single;


begin
  R := ClientRect;
  Canvas.Brush.Color := Color;
  Canvas.FillRect(R);
  for I := 1 to BorderWidth do AddEdge;
  if FPosition <= FMin then Exit;
  case FOrientation of
    toLeft, toRight: begin
      if FPosition < FMax then begin
        xD := R.Right - R.Left;
        if xD <= 0 then Exit;
        D := Round(xD * (FPosition - FMin) / (FMax - FMin));
        if D = 0 then
          Inc(D)
        else if D = xD then
          Dec(D)
        ;
        if FOrientation = toRight then
          R.Right := R.Left + D
        else
          R.Left := R.Right + D
        ;
      end;

      D := R.Bottom - 1 - R.Top;
      for I := 0 to D do begin
        F := I * 4 / D;
        xD := Floor(F);
        F := F - xD;
        if F = 0 then begin
          Canvas.Pen.Color := FTint[xD];
          Canvas.MoveTo(R.Left, R.Top + I);
          Canvas.LineTo(R.Right, R.Top + I);
        end
        else begin
          Canvas.Pen.Color := Fade(FTint[xD], FTint[xD + 1], F);
          Canvas.MoveTo(R.Left, R.Top + I);
          Canvas.LineTo(R.Right, R.Top + I);
        end;
      end;
    end;
    toUp, toDown: begin
      if FPosition < FMax then begin
        yD := R.Bottom - R.Top;
        if yD <= 0 then Exit;
        D := Round(yD * (FPosition - FMin) / (FMax - FMin));
        if D = 0 then
          Inc(D)
        else if D = yD then
          Dec(D)
        ;
        if FOrientation = toDown then
          R.Bottom := R.Top + D
        else
          R.Top := R.Bottom - D
        ;
      end;

      D := R.Right - 1 - R.Left;
      for I := 0 to D do begin
        F := I * 4 / D;
        yD := Floor(F);
        F := F - yD;
        if F = 0 then begin
          Canvas.Pen.Color := FTint[yD];
          Canvas.MoveTo(R.Left + I, R.Top);
          Canvas.LineTo(R.Left + I, R.Bottom);
        end
        else begin
          Canvas.Pen.Color := Fade(FTint[yD], FTint[yD + 1], F);
          Canvas.MoveTo(R.Left + I, R.Top);
          Canvas.LineTo(R.Left + I, R.Bottom);
        end;
      end;
      //for I := R.Top to
      //Canvas.Brush.Color := FTint;
      //Canvas.FillRect(R);

    end;
  end;
end;

procedure TTintProgress.SetBorderWidth(Value: Integer);
begin
  if Value <> FBorderWidth then begin
    FBorderWidth := Value;
    Invalidate;
  end;
end;

procedure TTintProgress.SetMax(Value: Integer);
begin
  if Value <> FMax then begin
    FMax := Value;
    Invalidate;
  end;
end;

procedure TTintProgress.SetMin(Value: Integer);
begin
  if Value <> FMin then begin
    FMin := Value;
    Invalidate;
  end;
end;

procedure TTintProgress.SetOrientation(Value: TOrientation);
begin
  if Value <> FOrientation then begin
    FOrientation := Value;
    Invalidate;
  end;
end;

procedure TTintProgress.SetPosition(Value: Integer);
begin
  if Value < FMin then
    Value := FMin
  else if Value > FMax then
    Value := FMax
  ;
  if Value <> FPosition then begin
    FPosition := Value;
    Invalidate;
  end;
end;

procedure TTintProgress.SetTint(Value: TColor);
var
  VB: TRGBFColor absolute Value;

begin
  if VB.F <> 0 then Value := GetSysColor(VB.R);

  if FSimpleTint and (Value = FTint[2]) then Exit;

  FTint[0] := Value;
  FTint[1] := $FFFFFF;
  FTint[2] := Value;
  FTint[3] := Fade(Value, 0, 1/3);
  FTint[4] := Fade(Value, 0, 2/3);
  FSimpleTint := True;

  Invalidate;
end;

procedure TTintProgress.SetTints(Index: Integer; Value: TColor);
begin
  if (Index < 0) or (Index > 4) then
    raise ERangeError.Create('Index ' + IntToStr(Index) + ' is invalid.')
  ;
  if FTint[Index] = Value then Exit;

  FTint[Index] := Value;
  FSimpleTint := False;

  Invalidate;
end;

end.